/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * NotMoleculeByIdAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "molecule.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "Descriptors/MoleculeIdDescriptor.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

#include "NotMoleculeByIdAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "NotMoleculeByIdAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
Action::state_ptr SelectionNotMoleculeByIdAction::performCall() {
  const molecule *mol = World::getInstance().getMolecule(MoleculeById(params.molindex.get()));
  if (mol != NULL) {
    if (World::getInstance().isSelected(mol)) {
      LOG(1, "Unselecting molecule " << mol->name);
      World::getInstance().unselectAllMolecules(MoleculeById(params.molindex.get()));
      LOG(0, World::getInstance().countSelectedMolecules() << " molecules remain selected.");
      return Action::state_ptr(new SelectionNotMoleculeByIdState(params));
    } else {
      return Action::success;
    }
  } else {
    return Action::failure;
  }
}

Action::state_ptr SelectionNotMoleculeByIdAction::performUndo(Action::state_ptr _state) {
  SelectionNotMoleculeByIdState *state = assert_cast<SelectionNotMoleculeByIdState*>(_state.get());

  World::getInstance().selectAllMolecules(MoleculeById(state->params.molindex.get()));

  return Action::state_ptr(_state);
}

Action::state_ptr SelectionNotMoleculeByIdAction::performRedo(Action::state_ptr _state){
  SelectionNotMoleculeByIdState *state = assert_cast<SelectionNotMoleculeByIdState*>(_state.get());

  World::getInstance().unselectAllMolecules(MoleculeById(state->params.molindex.get()));

  return Action::state_ptr(_state);
}

bool SelectionNotMoleculeByIdAction::canUndo() {
  return true;
}

bool SelectionNotMoleculeByIdAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
